
#ifndef KORNERS_GAME_ENGINE_AI
#define KORNERS_GAME_ENGINE_AI

#include "korners_game_engine.h"

#include <list>
#include <map>


class KornersGameEngineAI : public KornersEngine
{
    Q_OBJECT
public:
    explicit KornersGameEngineAI();
    void makeNextMove( const QPoint& src, const QPoint& dst );

    struct MoveInfo
    {
        QPoint begin;
        QPoint end;
        int profit;

        MoveInfo( QPoint pnt1, QPoint pnt2 )
        {
            begin = pnt1;
            end = pnt2;
        }
                
        MoveInfo( QPoint pnt1, QPoint pnt2, int p )
        {
            begin = pnt1;
            end = pnt2;
            profit = p;
        }

        bool operator < ( const MoveInfo& mvInf ) const
        {
            return profit < mvInf.profit; 
        }

        bool operator > ( const MoveInfo& mvInf ) const
        {
            return profit > mvInf.profit; 
        }
    };

private:
    /**
     *  All computer caps positions
     */
    QList<QPoint> m_capPositions;
    /**
     *  Values for all cells
     */
    static const char static_values_begin[];
    static const char static_values_end[];
    /**
     *  Value of current position
     */
    int m_curPositionVal;
    /**
     *  If the game is close to the end
     */
    bool m_endSpiel;
    /**
     *  Find the best move for computer
     */
    MoveInfo search( int depth );
    bool innerSearch( int* diff, int depth, owner tp );
    /**
     *  Positions sctatic value
     */
    inline int calcPosStaticValue( owner tp ) const
    {
        int summ = 0;
        const char* static_values = 0;
        if(m_endSpiel)
        {
            static_values = static_values_end;
        }
        else
        {
            static_values = static_values_begin;
        }

        for( int i = 0; i < FIELD_SIZE*FIELD_SIZE; i++ )
        {   
            if( m_field[i] == tp )
            {
                if( tp == opponent)
                    summ += static_values[i];
                if( tp == player )
                    summ += static_values[  FIELD_SIZE - i%FIELD_SIZE  - 1 + 
                                           (FIELD_SIZE - i/FIELD_SIZE - 1)*FIELD_SIZE ];
            }
        }

        return summ;
    }
    /**
     *  Calc all the caps at the enemy base
     */
    inline int calcEnemyBaseCaps( owner tp )
    {
        int how_much = 0;
        for(int i = 0; i < N_CAPS/2; i++)
        {
            if(tp == opponent)
            {
                if( m_field[ i%IN_ROW + IN_ROW + ( i/IN_ROW + FIELD_SIZE - N_ROWS )*FIELD_SIZE ] == opponent )
                    how_much++;
            }
            else if(tp == player)
            {
                if( m_field[ i%IN_ROW + ( i/IN_ROW )*FIELD_SIZE ] == player )
                    how_much++;
            }
        }

        return how_much;
    }

    inline void makeMove( QPoint src, QPoint dst )
    {
        applyMove( src, dst );
        int i = m_capPositions.indexOf( src );
        m_capPositions[i] = dst;
    }

    /**
     *  Mid value of all computer caps coordinates
     */
    inline QPoint calcMidPosition()
    {
        QPoint mid( 0, 0 );
        for( int i = 0; i < N_CAPS/2; i++ )
            mid += m_capPositions.at(i);

        mid /= N_CAPS/2;

        return mid;
    }
    /**
     *  Penalty for backward caps
     */
    inline int backwardPenalty( QPoint cap, QPoint mid )
    {
        if( ( cap.x() > 5) && ( cap.y() > 5 ) )
            return 0;
        QPoint diff = cap - mid;

        int deviation = diff.manhattanLength();
        if( deviation > 8 )
            deviation *= 5;
        return deviation*2;
    }
    /**
     *  Bonus for cap grouping
     */
    inline int groupingBonus()
    {
        int bonus = 0;
        QPoint mid = calcMidPosition();
        for( int i = 0; i < N_CAPS/2; i++ )
            bonus -= backwardPenalty( m_capPositions.at(i), mid );

        return bonus;
    }

    inline int moveValue( QPoint src, QPoint dst )
    {
        return ( dst.x() - dst.x() + dst.y() - src.y() )*2;
    }

    /**
     *  Take empty cells from reachable
     *  We need to do this cause markReachable returns not only empty cells
     */
    inline void pointsToGo( const bool* from_where, QVector<int>& ret ) const
    {
        ret.clear();
        for( int i = 0; i < FIELD_SIZE*FIELD_SIZE; i++ )
        {
            if( ( from_where[i] == true ) && ( m_field[i] == empty ) )
                ret.push_back(i);
        }
    }
    /**
     *  Compute desireability for available moves
     */
    int scoreAvailableMoves( std::list<MoveInfo>& moves, owner tp ); 
    int getScores( std::multimap<int, int>& all, std::list<MoveInfo>& ret, owner tp );
};

#endif
